import React, {FC, FormEvent, useEffect, useState} from "react";
import {Field, InjectedFormProps, reduxForm} from "redux-form";

import {useAppSelector} from "../../hooks/redux";

import {StyledSwitch} from "../StyledSwitch/StyledSwitch";
import {StyledInput} from "../StyledInput/StyledInput";
import {CustomTooltip} from "../CustomTooltip/CustomTooltip";

import {number, required} from "../../validators/validators";

import './controlForm.scss'

interface FormProps {
    sumType: string
    salary: number
    ndfl: boolean
}

type DataType = Record<string, {
    value: string;
    label: string;
    tooltip: boolean;
    calculateFunction: {
        multiplication: (a: number, b: number) => number;
        division: (a: number, b: number) => number;
    }
}>


const dataArr: DataType = {
    '1': {
        value: '1',
        label: 'Оклад за месяц',
        tooltip: false,

        calculateFunction: {
            multiplication: () => {
                return 0
            },
            division: (salary, divider) => {
                return divider === 3 ? salary / 160 : salary / 20
            }
        }
    },
    '2': {
        value: '2',
        label: 'МРОТ',
        tooltip: true,
        calculateFunction: {
            multiplication: () => {
                return 0
            },
            division: () => {
                return 0
            }
        }
    },
    '3': {
        value: '3',
        label: 'Оплата за день',
        tooltip: false,
        calculateFunction: {
            multiplication: (salary) => {
                return salary * 20
            },
            division: (salary) => {
                return   salary / 8
            }
        }
    },
    '4': {
        value: '4',
        label: 'Оплата за час',
        tooltip: false,
        calculateFunction: {
            multiplication: (salary, multiplier) => {
                return multiplier === 3 ? salary * 8 * 20 : salary * 8
            },
            division: () => {
                return 0
            }
        }
    },
};


let ControlForm: FC<InjectedFormProps<FormProps>> = (props) => {

    const {control} = useAppSelector(state => state.form);
    const [, setFormValue] = useState<string>(control?.values?.sumType);

    const handlerSubmit = (e: FormEvent) => {
        e.preventDefault()
    };



    useEffect(() => {
        setFormValue((prevSumType) => {

            const sumType = control?.values?.sumType;

            if (control?.values?.sumType === '2')
                return prevSumType;

            if (prevSumType < control?.values?.sumType) {

                const currentSallary = dataArr[prevSumType].calculateFunction.division(control?.values?.salary, sumType - Number(prevSumType));
                props.change('salary', currentSallary);
                return sumType
            } else if (prevSumType > control?.values?.sumType) {

                const currentSallary = dataArr[prevSumType].calculateFunction.multiplication(control?.values?.salary, Number(prevSumType) - sumType);
                props.change('salary', currentSallary);
                return sumType
            }

            return prevSumType
        });


    }, [control?.values?.sumType]);


    function getSalary(salary: number | string, ndfl: boolean): number {
        return ndfl ? Math.round(Number(salary)) : Math.round(Number(salary) * 0.87)
    }

    function getNdflPercent(salary: number | string, ndfl: boolean): number {
        return ndfl ? Math.round(Number(salary) * 100 / 87 - Number(salary)) : Math.round(Number(salary) * 0.13)
    }

    function getFullSalary(salary: number | string, ndfl: boolean): number {
        return ndfl ? Math.round(Number(salary) * 100 / 87) : Math.round(Number(salary))
    }


    return (
        <>
            <form onSubmit={handlerSubmit} className='controlForm'>
                <h5 className='controlForm__title'>Сумма</h5>
                {Object.entries(dataArr).map(([key, i],) =>
                    <label key={`controlForm__radioButton${key}`} className='controlForm__radioButton'>
                        <Field
                            component={'input'}
                            type='radio'
                            value={i.value}
                            name='sumType'
                        />
                        {i.label}
                        {i.tooltip && <CustomTooltip/>}

                    </label>
                )}


                <label className='controlForm__inputWrapper'>
                    <span className='title'>Указать с НДФЛ</span>
                    <Field
                        style={{margin: '10px'}}
                        component={StyledSwitch}
                        defaultChecked={true}
                        name='ndfl'
                        label={'Без НДФЛ'}
                    />

                </label>
                <label className='controlForm__inputWrapper'>

                    <Field
                        validate={[required, number]}
                        component={StyledInput}
                        name='salary'
                        type={'number'}
                    />
                    <span className='currency'>₽</span>

                </label>

            </form>
            {control?.values?.sumType == 1 &&
            <div className='calculateHint'>
              <p>
                <span
                    className='calculateHint__value'>{getSalary(control.values.salary, control.values.ndfl)}
                </span>
                сотрудник будет получать на руки</p>
              <p>
                <span
                    className='calculateHint__value'>{getNdflPercent(control.values.salary, control.values.ndfl)}
              </span>
                НДФЛ 13 % от оклада</p>
              <p>
                <span
                    className='calculateHint__value'>{getFullSalary(control.values.salary, control.values.ndfl)}
                </span>
                сотрудник будет получать на руки</p>
            </div>
            }
        </>
    )
};


export const ControlReduxForm = reduxForm({
    initialValues: {
        sumType: '1',
        salary: 40000,
        ndfl: true
    },
    form: 'control'
})(ControlForm);
